<?php
/**
 * Prints an accessibility-friendly link to edit a post or page.
 *
 * This also gives us a little context about what exactly we're editing
 * (post or page?) so that users understand a bit more where they are in terms
 * of the template hierarchy and their content. Helpful when/if the single-page
 * layout with multiple posts/pages shown gets confusing.
 *
 * @since Reef 0.1
 * @return void
 */
if ( ! function_exists( 'reef_edit_link' ) ) :

function reef_edit_link( $echo = true ) {

	if( ! $echo ){
		ob_start();
	}

	edit_post_link(
		sprintf(
			# translators: %s: Name of current post
			__( 'EDIT<span class="screen-reader-text"> "%s"</span>', 'reef' ),
			get_the_title()
		),
		'<span class="edit-link">',
		'</span>'
	);

	if( ! $echo ){
		$output = ob_get_contents();
		ob_end_clean();
		return $output;
	}
}
endif;

/**
 * Prints Author's name with a link, posted date and number of Total Comments.
 *
 * @since  Reef 0.1
 * @return void
 */

if ( ! function_exists( 'reef_entry_footer' ) ) :
	
	function reef_entry_footer() {

		# Get the author name; wrap it in a link.
		?>
		<footer class="post-footer">
            <div class="detail">

            	<!-- Hide this section in single page  -->
            	<?php ! is_single() && ! is_author() ? reef_posted_by() : '' ; ?>
				
				<?php reef_time_link(); ?>
				
				<span class="comment-link">
					<a href="<?php comments_link(); ?>">

						<?php printf( esc_html( '%d' ), absint( wp_count_comments( get_the_ID() )->approved ) );?>
					</a>
				</span>

				<?php if( is_single() && 'post' == get_post_type() ): ?>

					<?php if( get_the_category_list() ): ?>
						<span class="divider">/</span>
						<span class="cat-links">
							<span class="screen-reader-text">
								<?php echo esc_html__( 'Categories', 'reef' ); ?>
							</span>
							<?php echo get_the_category_list( ', ' ); ?>
						</span>
					<?php endif; ?>

					<?php if( get_the_tag_list() ): ?>
						<span class="divider">/</span>
						<span class="tag-links">
							<span class="screen-reader-text">
								<?php echo esc_html__( 'Tags', 'reef' ); ?>
							</span>
							<?php echo get_the_tag_list( '', ', ' ); ?>
						</span>
					<?php endif; ?>

				<?php endif; ?>
			</div>
		</footer>
	<?php
	if( ! is_single() ){
		echo '</div>'; # .post-content-inner-wrap
	}
	}
endif;

/**
 * Prints Posts title with link, edit link and a category
 *
 * @since  Reef 0.1
 * @uses   reef_edit_link()
 * @return void
 */
if ( ! function_exists( 'reef_entry_header' ) ) :

function reef_entry_header() {

	if( ! is_single() ){ 
		$cat = reef_get_the_category();
		$edit_link = reef_edit_link( false );
	?>
		<div class="post-content-inner-wrap">
			<header class="post-title">
			<?php if ( ! empty( $cat ) ): ?>
				<span class="cat">
					<?php
					echo sprintf( '<a href="%s" >%s</a>',
							esc_url( get_category_link( $cat[ 0 ]->term_id ) ),
							esc_html( $cat[ 0 ]->name )
						);
					echo $edit_link;
					?>
				</span>
			<?php endif; ?>
			<?php
			if( 'post' == get_post_type() )
				$edit_link = '';

			the_title( '<h2><a href="' . esc_url( get_permalink() ) . '" rel="bookmark">', '</a>' . $edit_link . '</h2>' );
			?>
			</header>
<?php
	}
}
endif;


/**
 * Prints featured image or dummy image if no featured image for posts
 *
 * @since  Reef 0.1
 * @param  array $args
 * @param  bool $show_eye whether to show eye icon while hover. default: true.
 * @return void
 */

if( ! function_exists( 'reef_post_thumbnail' ) ):

	function reef_post_thumbnail( $args ){

		$defaults = array(
			'size'      => 'large',
			'show_eye'  => true,
			'dummy'     => true,
			'permalink' => true
		);

		$args = wp_parse_args( $args, $defaults );
		# Don't print even the div when no thumbnail and dummy is disabled
		if( '' == get_the_post_thumbnail() && !$args[ 'dummy' ] ){
			return;	
		} 
		?>
		<div class="post-thumb">
		    <figure>
	    	<?php 
		    	if (  '' !== get_the_post_thumbnail() ){
		    		the_post_thumbnail( $args[ 'size' ] ); 
		    	}else{

		    		# Returns the placeholder image after escaping it
		    		# @uses esc_url()
		    		
		    		$feat_image_url = reef_get_dummy_image( array(
		    			'size' => $args[ 'size' ]
		    		) );
		    		echo sprintf( '<img src="%s" >', $feat_image_url ); 
		    	}
	    	?>
	    	<?php if( $args[ 'permalink' ] ): ?>
		        <a href="<?php the_permalink(); ?>">
	        		<?php if( $args[ 'show_eye' ] ): ?>
		            	<i class="fa fa-eye"></i>
	    			<?php endif; ?>
		        </a>
	    	<?php endif; ?>
		    </figure>
		</div>
		<?php
	}
endif;

/**
* Prints out the title and breadcrumb with background
*
* @since  Reef 0.1
* @uses   blogy_breadcrumb()
* @return void
*/
if( ! function_exists( 'reef_inner_banner' ) ):

function reef_inner_banner(){

	$bg_image = reef_get_option( 'inner_banner' );
	if( empty( $bg_image ) ){

		$bg_image = apply_filters ( 'reef_default_banner_image', get_template_directory_uri(). '/assets/images/inner-banner.jpg' );
	}
	?>
	<section class="wrapper wrap-inner-banner" style="background-image: url('<?php echo esc_url( $bg_image ); ?>">

		<header class="page-header">
			<?php 
			if( is_archive() ){
				the_archive_title( '<h1 class="page-title">','</h1>' );
				the_archive_description('<div class="page-description">','</div>');
			}elseif( is_single() || is_page() ){
				the_title( '<h1 class="page-title">','</h1>' );
			}elseif( is_search() ){
				echo '<h1 class="page-title">'. esc_html__( 'Search Results for: ', 'reef' ) . '<span>' . get_search_query() . '</span></h1>';
			}elseif( is_404() ){
				echo '<h1 class="page-title">'.esc_html__( 'Page Not Found', 'reef' ). '</h1>';
			}else{
				echo '<h1 class="page-title">'.esc_html__( 'Blog', 'reef' ). '</h1>';
			}
			reef_breadcrumb();
			?>
		</header>
	</section>
	<?php
}
endif;

/**
 * Prints HTML with author avatar, name.
 * 
 *  @since Reef 0.1
 *  @uses reef_time_link();
 *  @return void
 */
if ( ! function_exists( 'reef_posted_on' ) ) :

function reef_posted_by() {
?>
	<span class="author">
	    <a href="<?php echo esc_url( get_author_posts_url( get_the_author_meta( 'ID' ) ) ); ?>">
	        <?php echo get_avatar( get_the_author_meta( 'ID' ), 32 ); ?>
	    </a>
	</span>
	<span class="author-name">
		<a href="<?php echo esc_url( get_author_posts_url( get_the_author_meta( 'ID' ) ) ); ?>">
			<?php echo get_the_author(); ?>
		</a>
	</span>
	<span class="divider">/</span>
<?php
}
endif;

/**
 * Gets a nicely formatted string for the published date.
 *
 * @since Reef 0.1
 * @return string
 */
if ( ! function_exists( 'reef_time_link' ) ) :

function reef_time_link() {

	$time_string = '<time class="entry-date published updated" datetime="%1$s">%2$s</time>';

	if ( get_the_time( 'U' ) !== get_the_modified_time( 'U' ) ) {
		$time_string = '<time class="entry-date published" datetime="%1$s">%2$s</time><time class="updated" datetime="%3$s">%4$s</time>';
	}
	
	$time_string = sprintf( $time_string,
		get_the_date( DATE_W3C ),
		get_the_date(),
		get_the_modified_date( DATE_W3C ),
		get_the_modified_date()
	);
?>
	<span class="screen-reader-text"><?php echo esc_html__( 'Posted on', 'reef' ); ?></span>
	<span class="posted-on">
		<a href="<?php echo esc_url( get_day_link( get_the_time('Y'), get_the_time('m'), get_the_time('d') ) ); ?>" rel="bookmark">
			<?php echo $time_string; ?>
		</a>
	</span>
	<span class="divider">/</span>
	<?php		
}
endif;
